/*
 * Copyright © 2014-2024 PDF Technologies, Inc. All Rights Reserved.
 *
 * THIS SOURCE CODE AND ANY ACCOMPANYING DOCUMENTATION ARE PROTECTED BY INTERNATIONAL COPYRIGHT LAW
 * AND MAY NOT BE RESOLD OR REDISTRIBUTED. USAGE IS BOUND TO THE ComPDFKit LICENSE AGREEMENT.
 * UNAUTHORIZED REPRODUCTION OR DISTRIBUTION IS SUBJECT TO CIVIL AND CRIMINAL PENALTIES.
 * This notice may not be removed from this file.
 *
 */

package com.compdfkit.flutter.compdfkit_flutter.plugin;


import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.CHECK_OWNER_UNLOCKED;
import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.CHECK_PASSWORD;
import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.CLOSE;
import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.GET_FILE_NAME;
import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.GET_PERMISSIONS;
import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.HAS_CHANGE;
import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.IS_ENCRYPTED;
import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.IS_IMAGE_DOC;
import static com.compdfkit.flutter.compdfkit_flutter.constants.CPDFConstants.ChannelMethod.OPEN_DOCUMENT;

import android.content.Context;
import android.net.Uri;
import android.text.TextUtils;
import androidx.annotation.NonNull;
import com.compdfkit.core.document.CPDFDocument;
import com.compdfkit.core.document.CPDFDocument.PDFDocumentError;
import com.compdfkit.tools.common.utils.threadpools.CThreadPoolUtils;
import com.compdfkit.ui.reader.CPDFReaderView;
import io.flutter.Log;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel.Result;

public class CPDFDocumentPlugin extends BaseMethodChannelPlugin{

  private String documentUid;

  private CPDFReaderView readerView;

  public CPDFDocumentPlugin(Context context,
      BinaryMessenger binaryMessenger, String documentUid) {
    super(context, binaryMessenger);
    this.documentUid = documentUid;
  }

  public void setReaderView(CPDFReaderView readerView) {
    this.readerView = readerView;
  }


  @Override
  public String methodName() {
    return "com.compdfkit.flutter.document_" + documentUid;
  }

  @Override
  public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
    if (readerView == null || readerView.getPDFDocument() == null){
      result.error("-1", "CPDFReaderView isnull or CPDFDocument is null", null);
      return;
    }
    CPDFDocument document = readerView.getPDFDocument();
    switch (call.method){
      case OPEN_DOCUMENT:
        String filePath = call.argument("filePath");
        String fileUri = call.argument("fileUri");
        String openPwd = call.argument("password");
        PDFDocumentError error;
        if (!TextUtils.isEmpty(filePath)){
          error = document.open(filePath, openPwd);
        } else {
          error = document.open(Uri.parse(fileUri), openPwd);
        }
        if (error == PDFDocumentError.PDFDocumentErrorSuccess){
          readerView.setPDFDocument(document);
        }
        result.success(error.ordinal());
        break;
      case GET_FILE_NAME:
        result.success(document.getFileName());
        break;
      case IS_ENCRYPTED:
        result.success(document.isEncrypted());
        break;
      case IS_IMAGE_DOC:
        CThreadPoolUtils.getInstance().executeIO(()->{
          boolean isImageDoc = document.isImageDoc();
          result.success(isImageDoc);
        });
        break;
      case GET_PERMISSIONS:
        result.success(document.getPermissions().id);
        break;
      case CHECK_OWNER_UNLOCKED:
        result.success(document.checkOwnerUnlocked());
        break;
      case CHECK_PASSWORD:
        String password = call.argument("password");
        boolean isOwnerPwd = call.argument("isOwnerPassword");
        result.success(document.checkPassword(password, isOwnerPwd));
      case CLOSE:
        document.close();
        result.success(true);
        break;
      case HAS_CHANGE:
        result.success(document.hasChanges());
        break;
      default:break;
    }
  }
}
